#!/usr/bin/env python3

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import os
import json
from glob import glob
import pandas as pd
import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
import pickle

class DataTreatment:
    def __init__(self, pdn_system: str, root_path: str,
                 keywords: list = [], metadata_columns: list = []) -> None:
        '''
          This object recieves the root path where all systems data is located
          as well as the PDN system, the it is able to append multiple paths into
          a single dataset.

          Parameters:
            root_path (str): The main location where every system data library is
            pdn_system (str): The PDN system you wish to work with:
                                's1', 's2', 's3p' or 's3s'
            keywords (list): The columns to keep during extraction
            metadata_columns (list): The columns to ignore when doing Machine Learning
        '''
        # TODO: ADD A FUNCTIONALITY TO ALSO WORKS WITH URLS

        # Define the system
        if type(pdn_system) == str:
            if pdn_system in ['s1', 's2', 's3p', 's3s']:
                self.pdn_system = pdn_system
            else:
                raise TypeError(
                    'Error on __init__ method:   Only values allowed for pdn_system are "s1", "s2", "s3p" or "s3s"')
        else:
            raise TypeError(
                'Error on __init__ method:   Only string type is allowed for pdn_system')

        # Define the root path
        if type(root_path) == str:
            self.root_path = root_path
        else:
            raise TypeError(
                'Error on __init__ method:   Only string type is allowed for root_path')

        self.metadata_columns = metadata_columns.copy()
        # list(set(keywords.copy() + metadata_columns.copy()))
        self.keywords = keywords.copy()

        self.last_data = pd.DataFrame({})
        self.tables = dict()
        self.table_history = []

        return None

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PRIVATE METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    def __getPaths(self, main_path: str) -> list:
        return glob(main_path+'/*', recursive=False)

    def __readJson(self, path: str) -> dict:
        with open(path, 'r') as f:
            dictionary = json.load(f)
        return dictionary

    def __normalizeStateName(self, string: str) -> str:
        return string.replace('SESNA', '').replace('_', '').replace(' ', '').lower()

    def __execTableMethod(self, method: str, input: list, output: str) -> None:
        self.tables[output] = eval("""self._{0}__{1}({2})""".format(
            __class__.__name__, method, input))
        self.last_data = self.tables[output].copy()
        self.table_history.append(output)
        return None

    def __filterKeywords(self, dictionary: dict) -> dict:
        keywords = list(self.keywords.copy()) + \
            list(self.metadata_columns.copy())
        filtered_keys = []
        for keyword in keywords:
            filtered_keys += list(filter(lambda x: (keyword in x)
                                  or (keyword == x), list(dictionary.keys())))
        filtered_keys = list(set(filtered_keys))
        return {k: dictionary[k] for k in filtered_keys}

    def __flattenDictionary(self, dictionary: dict,
                            separator: str = '_', prefix: str = '') -> dict:
        # Source: https://www.geeksforgeeks.org/python-convert-nested-dictionary-into-flattened-dictionary/
        return {prefix + separator + str(k) if prefix else str(k): v
                for kk, vv in dictionary.items()
                for k, v in self.__flattenDictionary(vv, separator, str(kk)).items()
                } if isinstance(dictionary, dict) else {prefix: dictionary}

    def __flattenStatesData(self, state_paths: list) -> pd.DataFrame:
        # TODO: REFACTOR AND MODULARIZE THIS CODE
        flattened_data = pd.DataFrame({})

        for state_path in state_paths:
            # Get the state name and normalize it
            state = self.__normalizeStateName(state_path.split('/')[-1])

            # Get every data file on the state
            single_files = self.__getPaths(state_path)
            state_list = []
            for single_file in single_files:

                # Read the json as a dictionary
                dictionary_file = self.__readJson(single_file)

                # Flatten the dictionary and add it to the whole states' list
                state_list += [self.__filterKeywords(self.__flattenDictionary(item))
                               for item in dictionary_file]

            # Normalize the state data
            state_data = pd.DataFrame(state_list)
            state_data['estado_declarante'] = state
            flattened_data = flattened_data.append(
                state_data, ignore_index=True)

        return flattened_data

    def __normalizeData(self, input_name: list) -> pd.DataFrame:
        # TODO: MAKE THIS CODE TO AUTOMATICALLY ITERATE UNTIL THERE'S NO INCREASE IN
        # COLUMN COUNT
        # Source: https://stackoverflow.com/questions/35491274/split-a-pandas-column-of-lists-into-multiple-columns
        input_table = self.tables[input_name[0]]

        output_table = pd.DataFrame({})
        for column in input_table.columns:
            # Check if any element in the column is a list
            if any(isinstance(item, list) for item in input_table[column].to_list()) or any(isinstance(item, np.ndarray) for item in input_table[column].to_list()):
                # Extend the list in a single column into multiple columns with a single element
                df_dummy = pd.DataFrame([pd.Series(x)
                                        for x in input_table[column]])
                df_dummy.columns = [
                    column+'_{}'.format(i+1) for i in df_dummy.columns]

                # If there are nested dictionaries inside the elements, it flattens them
                df_dummy_list = df_dummy.to_dict('records')
                for i in range(len(df_dummy_list)):
                    df_dummy_list[i] = self.__flattenDictionary(
                        df_dummy_list[i])
                df_dummy = pd.DataFrame(df_dummy_list)

            # If there's no list in the dataframe column, read the column
            else:
                df_dummy = input_table[column]

            # Concat the treated columns into the output
            output_table = pd.concat([output_table, df_dummy], axis=1)
        return output_table

    def __missingTreatment(self, input_name: list) -> pd.DataFrame:
        # TODO: INCLUDE PERSONALIZED MISSING TREATMENT WITH MORE INTERPRETABILITY
        input_table = self.tables[input_name[0]]
        return input_table.fillna(self.missing_input)

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PUBLIC METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    def extractData(self, output_table: str = 'flattened_data') -> None:
        # The main path of the PDN system
        system_root_path = os.path.join(self.root_path, self.pdn_system)

        # The list of every state on the PDN system
        state_paths = self.__getPaths(system_root_path)

        # Flatten all the states' data and append it into a single dataframe
        self.__execTableMethod(method='flattenStatesData',
                               input=state_paths, output=output_table)

        return None

    def defineId(self, id_columns: list = ['CURP'], input_table: str = 'last_data',
                 output_table: str = 'id_data') -> None:
        # This method takes a list of columns from which to make an unique id

        # TODO: DELETE THIS DISCLAIMER WHEN RAN ON AN ENVIRONMENT WITH DATAT THAT
        #        SHOWS THE CURP,SUCH AS INSIDE THE PLATAFORMA DIGITAL NACIONAL
        print('''Note on defineId method
    NOTE: IDEALLY, THIS FUNCTION SHOULD TAKE THE CURP OF EACH PUBLIC SERVANT
    THIS IS IN ORDER TO HAVE A TRULY UNIQUE AND TRACEABLE IDENTIFIER
    FOR EVERYONE. HOWEVER, IN THIS EXERCISE WE'LL TAKE A COMBINATION OF FULL NAME
    PLUS THE RESPECTIVE GOVERNMENT ORGANISATION THE PERSON WORKS FOR
    PLUS THE STATE THAT REPORTED THE DATA.''')

        # Create a unique string for each public servant
        df = self.tables[input_table].copy()
        df[id_columns] = df[id_columns].astype(str)
        df['unique_id'] = df[id_columns].agg('-'.join, axis=1)
        df['unique_id'] = df['unique_id'].apply(
            lambda x: x.lower().replace(' ', '_'))

        self.tables[output_table] = df.copy()
        self.metadata_columns += id_columns

        return None

    def normalizeData(self, input_table: str = 'last_data',
                      output_table: str = 'normalized_data') -> None:
        # This method extends every element  with a list within the dataframe
        # into multiple columns in the same dataframe
        self.__execTableMethod(method='normalizeData',
                               input=[input_table], output=output_table)
        return None

    def missingData(self, input: any = 0, input_table: str = 'last_data',
                    output_table: str = 'missing_data') -> None:
        # This method turns all missing data into a value
        self.missing_input = input
        self.__execTableMethod(method='missingTreatment',
                               input=[input_table], output=output_table)
        return None


class FeatureEngineering(DataTreatment):

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PRIVATE METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------

    def _tokenize_tfidf(self, corpus: list) -> object:
        # To tokenize with tfidf

        # Define and fit the tokenizer
        tokenizer = TfidfVectorizer()
        tokenizer.fit(corpus)

        return tokenizer

    # TODO: CORRECTLY FIX THIS CLASS INHERITANCE PROBLEM
    def __execTableMethod(self, method: str, input: list, output: str) -> None:
        self.tables[output] = eval("""self._{0}__{1}({2})""".format(
            __class__.__name__, method, input))
        self.last_data = self.tables[output].copy()
        self.table_history.append(output)
        return None

    def __tokenizeData(self, input_name: list):
        # TODO: BASED UPON THE WORD VARIETY, THE ALGORITHM DETERMINES TO TOKENIZE
        # WITH A DICTIONARY OR WITH TF-IDF

        # le paso el nombre de las columnas con las que voy a armar el id
        # -> de hecho, el id se puede armar desde el extract

        # la tokenizacion si debe ser sobre todas las palabras del dataset

        # Read the table, define the features to ignore and the date_column to sort by
        input_table = self.tables[input_name[0]]
        output_table = input_table.copy()
        ignore_features = self.metadata_columns.copy() + ['unique_id']

        # Get the non-ignorable features
        iterable_features = [
            column for column in input_table.columns if column not in ignore_features]

        # Construct the word corpus over all strings on each column
        if self.column_corpus:
            for column in iterable_features:
                word_corpus = [
                    item for item in input_table[column] if type(item) == str]

                try:
                    # Fit the tokenizer
                    self.tokenizers[column] = self._tokenize_tfidf(word_corpus)

                    # Apply element-wise tokenization on all iterable_features
                    output_table[column] = input_table[column].apply(lambda x: list(
                        self.tokenizers[column].transform([x]).toarray()[0]) if type(x) == str else x)
                except:
                    self.metadata_columns.append(column)

        # Construct the word corpus over all strings found on the dataframe
        #     (a good approach if you have A LOT of computing power and storage)
        else:
            word_corpus = []
            for column in iterable_features:
                filtered_list = [
                    item for item in input_table[column] if type(item) == str]
                word_corpus += filtered_list

            try:
                # Fit the tokenizer
                self.tokenizers['all'] = self._tokenize_tfidf(word_corpus)

                # Apply element-wise tokenization on all iterable_features
                output_table[iterable_features] = input_table[iterable_features].applymap(
                    lambda x: self.tokenizers[column].transform([x]).toarray()[0] if type(x) == str else x)
            except:
                self.metadata_columns += iterable_features

        return output_table

    def __groupData(self, input_name: list):
        # Read the table, define the features to ignore and the date_column to sort by
        input_table = self.tables[input_name[0]]
        ignore_features = self.metadata_columns.copy()
        date_column = grouping_date_dictionary[self.pdn_system]

        # Get the non-ignorable features
        iterable_features = [
            column for column in input_table.columns if column not in ignore_features]

        # Sort the table and then group by unique_id
        sorted_table = input_table[iterable_features+[date_column]
                                   ].sort_values(by=date_column, ascending=True)
        grouped_table = sorted_table.groupby('unique_id').agg(
            ['min', 'max', 'median', 'mean', 'prod', 'sum', 'std', 'var', 'sem', 'count'])

        # Flatten index on columns
        new_columns = ['_'.join(t) for t in grouped_table.columns]
        grouped_table.columns = new_columns
        grouped_table.reset_index(inplace=True)

        return grouped_table.rename(columns={'index': 'unique_id'})

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PUBLIC METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------

    def tokenizeData(self, column_corpus: bool = True, input_table: str = 'last_data', output_table: str = 'tokenized_data') -> None:
        # This method tokenizes text data based on the bag of words of each column
        self.tokenizers = dict()
        self.column_corpus = column_corpus
        self.__execTableMethod(method='tokenizeData',
                               input=[input_table], output=output_table)
        return None

    def groupData(self, input_table: str = 'last_data', output_table: str = 'grouped_data') -> None:
        # This method groups data into multiple features
        self.__execTableMethod(method='groupData',
                               input=[input_table], output=output_table)
        return None


def main(root_path = './'):

    # The metadata column for every system
    metadata_columns = ['id']

    # For system 1
    metadata_items = ["tipo", "declaracionCompleta",
                      "actualizacionConflictoInteres", "actualizacion",
                      "institucion"]
    declaracion_items = ["nombre", "primerApellido", "segundoApellido",
                         "correoElectronico_institucional"]

    metadata_columns += ['metadata_'+item for item in metadata_items]
    metadata_columns += ['declaracion_situacionPatrimonial_datosGenerales_' +
                         item for item in declaracion_items]

    # For system 2
    metadata_columns += declaracion_items

    # For system 3s
    servidor_items = ["nombres", "primerApellido", "segundoApellido",
                      "genero_clave", "genero_valor"]

    metadata_columns += ["fechaCaptura",
                         "expediente", "fechaCaptura", "expediente"]
    metadata_columns += ['servidorPublicoSancionado_' +
                         item for item in servidor_items]

    # The unique build for ids in each system (at least in this exercise, without CURP)

    # For system 1, it will be the full name plus the delcaring institution plus estado_declarante
    declaracion_items = ["nombre", "primerApellido", "segundoApellido"]
    s1_id = ['declaracion_situacionPatrimonial_datosGenerales_' +
             item for item in declaracion_items]
    s1_id += ['estado_declarante', 'metadata_institucion']

    # For system 2, it will be similar
    s2_id = ["nombres", "primerApellido", "segundoApellido"]
    s2_id += ['estado_declarante', 'institucionDependencia_siglas']

    # For system 3, it is only the full name and estado_declarante because there's
    # no info about the institution the server works for
    servidor_items = ["nombres", "primerApellido", "segundoApellido"]

    s3s_id = ['servidorPublicoSancionado_'+item for item in servidor_items]
    s3s_id += ['estado_declarante']

    # Put every list into a single dictionary
    systems_id_dictionary = {'s1': s1_id, 's2': s2_id, 's3s': s3s_id}

    grouping_date_dictionary = {'s1': 'metadata_actualizacion',
                                's2': 'fechaCaptura', 's3s': 'fechaCaptura', 's3p': 'fechaCaptura'}

    # SINCE THERE'S A LOT OF RAM REQUIREMENTS, WE DROP HIGH NAN COLUMNS AND LOW VARIANCE ONES
    for system in ['s1', 's2', 's3s', 's3p']:
        if system == 's1':
            etl = FeatureEngineering(pdn_system=system, root_path=root_path,
                                     keywords=['escolaridad', 'ingreso', 'inversion'], metadata_columns=metadata_columns)

            # Initial extraction and normalization
            etl.extractData('ext_data')

            # Define the unique id of each servant
            etl.defineId(systems_id_dictionary[system], 'ext_data', 'ext_data')

            etl.normalizeData('ext_data', 'norm_data_inter_1')
            etl.normalizeData('norm_data_inter_1', 'norm_data_inter_2')
            etl.normalizeData('norm_data_inter_2', 'norm_data_1')

            # Treat all missing data
            etl.missingData(0, 'norm_data_1', 'miss_data_1')

            # Save this untokenized ungrouped data
            etl.last_data.to_csv(root_path+system+'/ut_ug_m_data.csv', index=False)

            # Group the historic data
            etl.groupData('miss_data_1', 'group_data_1')

            # Save this untokenized grouped data
            etl.last_data.to_csv(root_path+system+'/ut_g_um_data.csv', index=False)

            # Missing treatment for new NaNs (it has a different missing input because the origin is different)
            etl.missingData(-1, 'group_data_1', 'miss_data_2')

            # Save this untokenized grouped data
            etl.last_data.to_csv(root_path+system+'/ut_g_m_data.csv', index=False)

            # Tokenize the text data and then normalize the newly created vectors
            etl.tokenizeData(True, 'miss_data_1', 'tok_data')
            etl.normalizeData('tok_data', 'norm_data_2')
            etl.missingData(0, 'norm_data_2', 'miss_data_3')
            etl.last_data.to_csv(root_path+system+'/t_ug_m_data.csv', index=False)

        else:
            etl = FeatureEngineering(pdn_system=system, root_path=root_path,
                                     keywords=['nivel', 'tipo', 'nombre', 'clave', 'siglas', 'valor', 'superior', 'expediente', 'objeto', 'acto', 'causa', 'responsable', 'resolucion', 'multa', 'observacion', 'documento'], metadata_columns=metadata_columns)

            # Initial extraction and normalization
            etl.extractData('ext_data')

            # Define the unique id of each servant
            if system != 's3p':
                etl.defineId(
                    systems_id_dictionary[system], 'ext_data', 'ext_data')

            etl.normalizeData('ext_data', 'norm_data_inter_1')

            etl.normalizeData('norm_data_inter_1', 'norm_data_inter_2')

            etl.normalizeData('norm_data_inter_2', 'norm_data_1')

            # Treat all missing data
            etl.missingData(0, 'norm_data_1', 'miss_data_1')

            # Save this untokenized ungrouped data
            etl.last_data.to_csv(root_path+system+'/ut_ug_m_data.csv', index=False)

            # Tokenize the text data and then normalize the newly created vectors
            etl.tokenizeData(True, 'miss_data_1', 'tok_data')

            etl.normalizeData('tok_data', 'norm_data_2')

            etl.missingData(0, 'norm_data_2', 'miss_data_3')

            # Save this tokenized ungrouped data
            etl.last_data.to_csv(root_path+system+'/t_ug_m_data.csv', index=False)

        with open(root_path+system+'/etl.pkl', 'wb') as f:
            pickle.dump(etl, f)


if __name__ == '__main__':
    main(root_path='./data/')



#!/usr/bin/env python3

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from setuptools import setup, find_packages
import os

root_dir = os.path.dirname(__file__)

REQUIRES = [
"asttokens==2.2.0",
"autopep8==2.0.0",
"backcall==0.2.0",
"bleach==5.0.1",
"build==0.9.0",
"certifi==2022.9.24",
"cffi==1.15.1",
"charset-normalizer==2.1.1",
"commonmark==0.9.1",
"contourpy==1.0.6",
"cryptography==38.0.4",
"cycler==0.11.0",
"debugpy==1.6.4",
"decorator==5.1.1",
"docutils==0.19",
"entrypoints==0.4",
"executing==1.2.0",
"fonttools==4.38.0",
"idna==3.4",
"importlib-metadata==5.1.0",
"ipykernel==6.17.1",
"ipython==8.7.0",
"jaraco.classes==3.2.3",
"jedi==0.18.2",
"jeepney==0.8.0",
"joblib==1.2.0",
"jupyter_client==7.4.7",
"jupyter_core==5.1.0",
"keyring==23.11.0",
"kiwisolver==1.4.4",
"matplotlib==3.6.2",
"matplotlib-inline==0.1.6",
"more-itertools==9.0.0",
"nest-asyncio==1.5.6",
"numpy==1.23.5",
"packaging==21.3",
"pandas==1.5.2",
"parso==0.8.3",
"pep517==0.13.0",
"pexpect==4.8.0",
"pickleshare==0.7.5",
"Pillow==9.3.0",
"pkg_resources==0.0.0",
"pkginfo==1.9.2",
"platformdirs==2.5.4",
"prompt-toolkit==3.0.33",
"psutil==5.9.4",
"ptyprocess==0.7.0",
"pure-eval==0.2.2",
"pycodestyle==2.10.0",
"pycparser==2.21",
"Pygments==2.13.0",
"pyparsing==3.0.9",
"python-dateutil==2.8.2",
"pytz==2022.6",
"pyzmq==24.0.1",
"readme-renderer==37.3",
"requests==2.28.1",
"requests-toolbelt==0.10.1",
"rfc3986==2.0.0",
"rich==12.6.0",
"scikit-learn==1.1.3",
"scipy==1.9.3",
"seaborn==0.12.1",
"SecretStorage==3.3.3",
"six==1.16.0",
"stack-data==0.6.2",
"threadpoolctl==3.1.0",
"tomli==2.0.1",
"tornado==6.2",
"traitlets==5.6.0",
"twine==4.0.2",
"typing_extensions==4.4.0",
"urllib3==1.26.13",
"wcwidth==0.2.5",
"webencodings==0.5.1",
"wget==3.2",
"zipp==3.11.0",
]

with open(os.path.join(root_dir, "README.md"), "r") as fh:
    long_description = fh.read()

setup(
    name="unblind",
    version="0.0.4",
    python_requires=">=3.8",
    description="Unblind is a Python package to create data visualizations from data of the Plataforma Digital Nacional, Sistema Nacional Anticorrupción",
    long_description=long_description,
    packages=find_packages(),
    long_description_content_type="text/markdown",
    install_requires=REQUIRES,
)




import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import wget
import zipfile
import os
import json

datasets_urls = {
's1': 'https://api.plataformadigitalnacional.org/bulks-conteos/download?sistema=s1&elemento=zip',
's2': 'https://api.plataformadigitalnacional.org/bulks-conteos/download?sistema=s2&elemento=zip',
's3s': 'https://api.plataformadigitalnacional.org/bulks-conteos/download?sistema=s3s&elemento=zip',
's3p': 'https://api.plataformadigitalnacional.org/bulks-conteos/download?sistema=s3p&elemento=zip',
}

def get_datasets(to_path = 'data/'):

    data = {
        's1': None,
        's2': None,
        's3s': None,
        's3p': None,
    }

    # Create data folder if it doesn't exist
    if not os.path.exists(to_path):
        os.makedirs(to_path)

    for key, url in datasets_urls.items():
        print('Loading dataset: {}'.format(key))
        # Download file
        filename = wget.download(url, out=to_path)

        # Create folder
        folder = os.path.join(to_path, key)
        if not os.path.exists(folder):
            os.makedirs(folder)

        # Unzip file
        with zipfile.ZipFile(filename, 'r') as zip_ref:
            zip_ref.extractall(folder)

        # Remove zip file
        os.remove(filename)




#!/usr/bin/env python3

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import pandas as pd
import numpy as np
import seaborn as sns

sns.set_theme(context='talk', style='darkgrid', palette='colorblind')

class DataViz:
    def __init__(self, pdn_system: str, root_path: str = './') -> None:
        self.pdn_system = pdn_system
        self.root_path = root_path
        return None

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PRIVATE METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    def __featureSanityCheck(self, input_table: pd.DataFrame,
                             thresh: int = 10) -> list:

        non_constant_columns = []
        for column in input_table:
            if len(input_table[column].value_counts()) >= thresh:
                non_constant_columns.append(column)

        return non_constant_columns

    def __bivariatePlot(self, group_data: list,
                        file_path: str, variables: list) -> object:
        input_table = pd.read_csv(
            self.root_path+self.pdn_system+'/'+file_path+'_data.csv', usecols=variables)

        if group_data[0] and group_data[1]:
            # Stacked countplot
            df_plot = input_table.groupby(variables).size().reset_index().pivot(
                columns=variables[0], index=variables[1], values=0)
            output_graph = df_plot.plot(kind='bar', stacked=True)

        elif not (group_data[0] or group_data[1]):
            # Scatter plot
            temp_input_table = input_table.copy()

            # Add noise to avoid overlapping points

            # Check if the variable is categorical
            if len(temp_input_table[variables[0]].value_counts()) < 10:
                temp_input_table[variables[0]] = temp_input_table[variables[0]] + \
                    np.random.normal(0, 0.1, len(temp_input_table))
            if len(temp_input_table[variables[1]].value_counts()) < 10:
                temp_input_table[variables[1]] = temp_input_table[variables[1]] + \
                    np.random.normal(0, 0.1, len(temp_input_table))

            # Check if any of the variables is categorical
            if len(input_table[variables[0]].value_counts()) < 10 or len(input_table[variables[1]].value_counts()) < 10:
                output_graph = sns.scatterplot(
                    data=temp_input_table, x=variables[0], y=variables[1], alpha=0.2, size = 0.2)
            else:
                output_graph = sns.scatterplot(
                    data=temp_input_table, x=variables[0], y=variables[1], alpha =0.7)
        else:
            # Boxplot
            if group_data[0]:
                df_plot = input_table.groupby(variables[0])
            else:
                df_plot = input_table.groupby(variables[1])
            output_graph = df_plot.boxplot(subplots=False)

        return output_graph

    def __univariatePlot(self, group_data: bool,
                         file_path: str, variable: str) -> object:
        input_series = pd.read_csv(
            self.root_path+self.pdn_system+'/'+file_path+'_data.csv', usecols=[variable])
        if group_data:
            # Pie chart with counts
            output_graph = input_series.value_counts().plot(kind='pie')
        else:
            # Histogram
            output_graph = sns.histplot(input_series)
        return output_graph

    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # PUBLIC METHODS
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    # ----------------------------------------------------------------------------
    def availableFeatures(self, path_list:list, thresh:int=10) -> None:

        path_available_features = dict()
        for path in path_list:
            path_table = pd.read_csv(
                self.root_path+self.pdn_system+'/'+path+'_data.csv')
            path_available_features[path] = self.__featureSanityCheck(
                path_table, thresh)

        self.path_available_features = path_available_features

        return None

    def createGraph(self, group_data: list, file_path: str, variables: list) -> object:

        if len(variables) == 2:
            output_graph = self.__bivariatePlot(
                group_data, file_path, variables)
        elif len(variables) == 1:
            output_graph = self.__univariatePlot(
                group_data[0], file_path, variables[0])
        else:
            raise Exception(
                'Error on createGraph method:   Input variables must be a list of strings with length 1 or 2')
        output_graph.set_xticklabels(output_graph.get_xticklabels(), rotation=90)
        return output_graph


# This function is for recommending a list of features
# (On stramlit app, it can show the displayed features to select)
def give_viable_features(root_path: str, system: str, aggregated_data: bool, thresh: int) -> tuple:
    system_paths = {'s1': ['ut_ug_m', 'ut_g_m'],
                    's2': ['ut_ug_m'],
                    's3s': ['ut_ug_m'],
                    's3p': ['ut_ug_m']}

    if system != 's1':
        aggregated_data = False

    path = 'ut_ug_m'
    if aggregated_data:
        path = 'ut_g_m'

    data_visualizer = DataViz(pdn_system=system, root_path=root_path)
    data_visualizer.availableFeatures(system_paths[system], thresh)

    # NOTE: THE FEATURES MIGHT NEED SOME TREATMENT IN ORDER TO PROPERLY SHOW THEM
    return path, data_visualizer.path_available_features[path]


# Based on the selected features and path, it makes the optimal graph
def graph_features(root_path: str, system: str, group_data: list, file_path: str, variables: list) -> object:
    # NOTE: group_data IS A LIST OF BOOLEANS
    data_visualizer = DataViz(pdn_system=system, root_path=root_path)

    return data_visualizer.createGraph(group_data, file_path, variables)

if __name__=='__main__':
    print('This is a module for data visualization of the PDN system data 🙌🏽')
